package com.yanzhu.web.controller.aibox;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.yanzhu.common.annotation.Anonymous;
import com.yanzhu.common.annotation.RateLimiter;
import com.yanzhu.common.core.controller.BaseController;
import com.yanzhu.common.core.domain.AjaxResult;
import com.yanzhu.common.core.domain.entity.SysDictData;
import com.yanzhu.common.core.page.TableDataInfo;
import com.yanzhu.common.core.redis.RedisCache;
import com.yanzhu.common.core.text.Convert;
import com.yanzhu.common.enums.LimitType;
import com.yanzhu.common.utils.DictUtils;
import com.yanzhu.common.utils.StringUtils;
import com.yanzhu.system.domain.ai.DevAiProjectData;
import com.yanzhu.system.domain.ai.DevAiProjectDataVO;
import com.yanzhu.system.service.IDevAiProjectDataService;
import com.yanzhu.system.service.ISysDictTypeService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * AIBoxController
 * 智能盒子
 * @author JiangYuQi
 * @date 2023-07-29
 */
@RestController
@RequestMapping("/publics/aiBox")
public class AIBoxController extends BaseController {

    @Autowired
    private IDevAiProjectDataService devAiProjectDataService;

    private static final Logger logger = LoggerFactory.getLogger("AIBoxController");

    /**
     * aibox推送预警数据
     */
    @Anonymous
    @PostMapping("/v1/push")
    @RateLimiter(count = 10, limitType = LimitType.IP)
    public void push(@RequestBody Map<String,Object> dataMap)
    {
        try{
            List<Map<String,Object>> list = (List<Map<String,Object>>)dataMap.get("BehaviorResults");
            DevAiProjectDataVO devAiProjectDataVO = JSON.parseObject(JSONObject.toJSONString(list.get(0)), DevAiProjectDataVO.class);
            devAiProjectDataService.insertHttpDevAiProjectData(devAiProjectDataVO);
        }catch (Exception e){
            logger.info("AiBox解析参数失败！！！");
            logger.info(dataMap.toString());
            e.printStackTrace();
        }

    }

    private final static Integer CACHE_TIME = 1;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysDictTypeService dictTypeService;

    /**
     * 查询字典类型详细
     */
    @Anonymous
    @GetMapping(value = "/dicts")
    public AjaxResult getDictType()
    {
        String key="PUBLIC_AIBOX_dicts";
        Object obj=redisCache.getCacheObject(key);
        if(obj!=null){
            return success(obj);
        }
        List<SysDictData> list = dictTypeService.selectDictDataByType("aibox_alarm_type");
        redisCache.setCacheObject(key, list, CACHE_TIME*5, TimeUnit.MINUTES);
        return success(list);
    }

    /**
     * 查询AI预警列表
     */
    @Anonymous
    @GetMapping("/v1/listView")
    public AjaxResult listView(String deptId, String projectId)
    {
        String key="PUBLIC_AIBOX_listView_"+deptId+"_"+projectId;
        Object obj=redisCache.getCacheObject(key);
        if(obj!=null){
            return success(obj);
        }

        DevAiProjectData devAiProjectData = new DevAiProjectData();
        if(StringUtils.isNotEmpty(deptId)){
            devAiProjectData.setDeptId(Convert.toLong(deptId));
        }
        if(StringUtils.isNotEmpty(projectId)){
            devAiProjectData.setProjectId(Convert.toLong(projectId));
        }
        //默认查询当天数据
        /*Map<String, Object> params = new HashMap<>();
        params.put("date",new Date());
        devAiProjectData.setParams(params);*/
        startPage();
        List<DevAiProjectData> list = devAiProjectDataService.selectDevAiProjectDataList(devAiProjectData);
        redisCache.setCacheObject(key, list, CACHE_TIME, TimeUnit.MINUTES);
        return success(list);
    }

    /**
     * 查询AI预警列表
     */
    @Anonymous
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody DevAiProjectData devAiProjectData)
    {
        startPage();
        List<DevAiProjectData> list = devAiProjectDataService.selectDevAiProjectDataList(devAiProjectData);
        return getDataTable(list);
    }

    /**
     * 查询视频配置列表
     */
    @Anonymous
    @GetMapping("/groupCountByAlarmType")
    public AjaxResult groupCountByAlarmType(String deptId,String projectId,String now)
    {
        String key="PUBLIC_AIBOX_groupCountByAlarmType_"+deptId+"_"+projectId+"_"+now;
        Object obj=redisCache.getCacheObject(key);
        if(obj!=null){
            return success(obj);
        }
        DevAiProjectData devAiProjectData = new DevAiProjectData();
        if(StringUtils.isNotEmpty(deptId)){
            devAiProjectData.setDeptId(Convert.toLong(deptId));
        }
        if(StringUtils.isNotEmpty(projectId)){
            devAiProjectData.setProjectId(Convert.toLong(projectId));
        }
        //查询当天数据
        if(now!=null && "Y".equals(now)){
            Map<String, Object> params = new HashMap<>();
            params.put("date",new Date());
            devAiProjectData.setParams(params);
        }
        List<Map<String, Object>> list = devAiProjectDataService.groupCountByAlarmType(devAiProjectData);
        redisCache.setCacheObject(key, list, CACHE_TIME, TimeUnit.MINUTES);
        return success(list);
    }

}
